@@ -145,10 +145,11 @@ void AvrDevice::Load(const char* fname) {
 }
 #endif
 
-void AvrDevice::SetClockFreq(unsigned long f) {
+void AvrDevice::SetClockFreq(SystemClockOffset f) {
     clockFreq=f;
 }
-unsigned long long AvrDevice::GetClockFreq() {
+
+SystemClockOffset AvrDevice::GetClockFreq() {
     return clockFreq;
 }
 
@@ -298,9 +299,10 @@ int AvrDevice::Step(bool &untilCoreStepFinished, SystemClockOffset *nextStepIn_n
                 if (cpuCycles<=0) {
                     if ((unsigned int)(PC<<1) >= (unsigned int)Flash->GetSize() ) {
                         if (trace_on) {
-                            traceOut << "Simulation runs out of Flash Space" << endl;
+                            traceOut << actualFilename << " Simulation runs out of Flash Space at" << hex << (PC << 1) << endl;
+                            traceOut.flush();
                         } else {
-                            cerr << "Simulation runs out of Flash Space" << endl;
+                            cerr << actualFilename << " Simulation runs out of Flash Space at " << hex << (PC << 1) << endl;
                         }
                         exit(0);
                     }
@@ -334,7 +336,11 @@ int AvrDevice::Step(bool &untilCoreStepFinished, SystemClockOffset *nextStepIn_n
         }
 
         cpuCycles--;
-        if (trace_on==1) traceOut << endl;
+        if (trace_on==1) {
+            traceOut << endl;
+            TraceNextLine();
+        }
+
         //if (untilCoreStepFinished == false) { //we wait not until end so reply the finish state
             untilCoreStepFinished= !((cpuCycles>0) || (hwWait>0));
             return bpFlag;
