@@ -47,7 +47,7 @@ using namespace std;
 
 class AvrDevice: public SimulationMember {
     protected:
-        unsigned long clockFreq;
+        SystemClockOffset clockFreq;
         map < string, Pin *> allPins; 
 
         //old static vars for Step()
@@ -85,14 +85,15 @@ class AvrDevice: public SimulationMember {
         void Load(const char* n);
 
         Pin *GetPin(const char *name);
-
+#ifndef SWIG
         MemoryOffsets &operator->*(MemoryOffsets *m) { return *m;}
+#endif
 
         AvrDevice(unsigned int flashSize, unsigned int IRamSize, unsigned int ERamSize, unsigned int IoSpaceSize);
         int Step(bool &, SystemClockOffset *nextStepIn_ns =0);
         void Reset();
-        void SetClockFreq(unsigned long f);
-        unsigned long long GetClockFreq();
+        void SetClockFreq(SystemClockOffset f);
+        SystemClockOffset GetClockFreq();
 
         virtual ~AvrDevice();
         virtual unsigned char GetRampz();
